@@ -1,9 +1,9 @@
-from __future__ import absolute_import, unicode_literals
+__all__ = ["Response"]
 
-__all__ = ['Response']
+from typing import Any, MutableMapping, Optional
 
 
-class Response(object):
+class Response:
     """HTTP response.
 
     :param method: HTTP method in lowercase (e.g. "post").
@@ -11,7 +11,7 @@ class Response(object):
     :param url: API URL.
     :type url: str
     :param headers: Response headers.
-    :type headers: requests.structures.CaseInsensitiveDict | dict
+    :type headers: MutableMapping
     :param status_code: Response status code.
     :type status_code: int
     :param status_text: Response status text.
@@ -24,7 +24,7 @@ class Response(object):
     :ivar url: API URL.
     :vartype url: str
     :ivar headers: Response headers.
-    :vartype headers: requests.structures.CaseInsensitiveDict | dict
+    :vartype headers: MutableMapping
     :ivar status_code: Response status code.
     :vartype status_code: int
     :ivar status_text: Response status text.
@@ -32,7 +32,7 @@ class Response(object):
     :ivar raw_body: Raw response body.
     :vartype raw_body: str
     :ivar body: JSON-deserialized response body.
-    :vartype body: str | bool | int | list | dict
+    :vartype body: str | bool | int | float | list | dict | None
     :ivar error_code: Error code from ArangoDB server.
     :vartype error_code: int
     :ivar error_message: Error message from ArangoDB server.
@@ -42,25 +42,27 @@ class Response(object):
     """
 
     __slots__ = (
-        'method',
-        'url',
-        'headers',
-        'status_code',
-        'status_text',
-        'body',
-        'raw_body',
-        'error_code',
-        'error_message',
-        'is_success',
+        "method",
+        "url",
+        "headers",
+        "status_code",
+        "status_text",
+        "body",
+        "raw_body",
+        "error_code",
+        "error_message",
+        "is_success",
     )
 
-    def __init__(self,
-                 method,
-                 url,
-                 headers,
-                 status_code,
-                 status_text,
-                 raw_body):
+    def __init__(
+        self,
+        method: str,
+        url: str,
+        headers: MutableMapping[str, str],
+        status_code: int,
+        status_text: str,
+        raw_body: str,
+    ) -> None:
         self.method = method.lower()
         self.url = url
         self.headers = headers
@@ -69,7 +71,7 @@ class Response(object):
         self.raw_body = raw_body
 
         # Populated later
-        self.body = None
-        self.error_code = None
-        self.error_message = None
-        self.is_success = None
+        self.body: Any = None
+        self.error_code: Optional[int] = None
+        self.error_message: Optional[str] = None
+        self.is_success: Optional[bool] = None
